#
# spec file for package feedbackd
#
# Copyright (c) 2025 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%define soname libfeedback-0_0-0

Name:           feedbackd
Version:        0.5.0
Release:        2.1
Summary:        Feedback library for GNOME
License:        GPL-3.0-only AND LGPL-2.1-only
URL:            https://source.puri.sm/Librem5/feedbackd
Source0:        %{name}-%{version}.tar.xz

BuildRequires:  c_compiler
BuildRequires:  dbus-1
BuildRequires:  meson
BuildRequires:  vala
BuildRequires:  pkgconfig(gio-2.0) >= 2.50.0
BuildRequires:  pkgconfig(gio-unix-2.0) >= 2.50.0
BuildRequires:  pkgconfig(glib-2.0) >= 2.50.0
BuildRequires:  pkgconfig(gmobile)
BuildRequires:  pkgconfig(gobject-2.0) >= 2.50.0
BuildRequires:  pkgconfig(gobject-introspection-1.0)
BuildRequires:  pkgconfig(gsound)
BuildRequires:  pkgconfig(gudev-1.0) >= 232
BuildRequires:  pkgconfig(json-glib-1.0)
BuildRequires:  pkgconfig(umockdev-1.0)

%description
feedbackd provides a DBus daemon (feedbackd) to act on events to provide
haptic, visual and audio feedback. It offers a library (libfeedback) and
GObject introspection bindings to ease using it from applications.

%package -n typelib-1_0-Lfb-0_0
Summary:        Introspection bindings for %{name}

%description -n typelib-1_0-Lfb-0_0
feedbackd provides a DBus daemon (feedbackd) to act on events to provide
haptic, visual and audio feedback. It offers a library (libfeedback) and
GObject introspection bindings to ease using it from applications.

This package contains the introspection bindings for %{name}.

%package -n %{soname}
Summary:        Shared library for %{name}

%description -n %{soname}
feedbackd provides a DBus daemon (feedbackd) to act on events to provide
haptic, visual and audio feedback. It offers a library (libfeedback) and
GObject introspection bindings to ease using it from applications.

This package contains the shared library for %{name}.

%package devel
Summary:        Development files for %{name}
Requires:       %{name} = %{version}
Requires:       %{soname} = %{version}
Requires:       typelib-1_0-Lfb-0_0 = %{version}

%description devel
The %{name}-devel package contains libraries and header files for
developing applications that use %{name}.

%prep
%autosetup -p1

%build
%meson -Dsystemd_user_unit_dir=%{_userunitdir} \
	%{nil}
%meson_build

%install
%meson_install

%pre
getent group feedbackd >/dev/null || groupadd -r feedbackd
exit 0

%check
%meson_test

%ldconfig_scriptlets -n %{soname}

%files
%{_bindir}/fbcli
%{_bindir}/fbd-theme-validate
%{_libexecdir}/feedbackd
%{_libexecdir}/fbd-alert-slider
%{_libexecdir}/fbd-ledctrl
%{_datadir}/dbus-1/interfaces/org.sigxcpu.Feedback.xml
%{_datadir}/dbus-1/services/org.sigxcpu.Feedback.service
%{_datadir}/feedbackd
%{_datadir}/glib-2.0/schemas/org.sigxcpu.feedbackd.gschema.xml
%{_udevrulesdir}/*
%{_userunitdir}/fbd-alert-slider.service

%files -n typelib-1_0-Lfb-0_0
%{_libdir}/girepository-1.0/Lfb-0.0.typelib

%files -n %{soname}
%{_libdir}/libfeedback-0.0.so.0

%files devel
%{_libdir}/libfeedback-0.0.so
%{_includedir}/libfeedback-0.0/
%{_datadir}/vala/
%{_datadir}/gir-1.0/Lfb-0.0.gir
%{_libdir}/pkgconfig/libfeedback-0.0.pc

%changelog
* Sat Nov 16 2024 Adrian Campos Garrido <adriancampos@teachelp.com>
- Update to version 0.5.0
  + Detect haptic on Google Sargo and Oneplus Enchilada/Fachita
  + Handle "alert slider" on Oneplus Enchilada/Fachita
* Fri Aug 30 2024 Bjørn Lie <bjorn.lie@gmail.com>
- Update to version 0.4.1:
  + Simplify udev rule installation
  + Dont' fail tests with newer json-glib
- Changes from version 0.4.0:
  + Support `hw_pattern` on devices using qcom-lpg. This allows to
    keep LEDs on in suspend.
  + Internal cleanups and restructuring to cater for different LED
    types.
  + Support RGB color mixing for multicolor leds
  + Allow to use (camera) flash LEDs as notificaton LED
  + Immediately end "too noisy" feedbacks when switching profile
    levels
- Changes from version 0.3.0:
  + Allow blessed apps (e.g. alarm clocks) to override the global
    feedback level
  + Documentation, CI and packaging updates
  + Make it simpler to add new led types
- Add pkgconfig(gmobile) and pkgconfig(umockdev-1.0) BuildRequires:
  New dependencies.
* Wed Sep  6 2023 Bjørn Lie <bjorn.lie@gmail.com>
- Update to version 0.2.1:
  + event: Fix doc link
  + treewide: Drop superfluous '()' after functions in doc strings.
    Otherwise we have that doubled in the docs.
  + doc:
  - Better cross reference manpages.
  - Add manpage for feedback themes. This should direct users to
    the documentation.
* Fri Jul 21 2023 Bjørn Lie <bjorn.lie@gmail.com>
- Update to version 0.2.0:
  + build: Bump version to 0.2.0. We have a slight api change.
  + lfb-event: Avoid allocation when getting profile. We can return
    the const *. This makes it consistent with other getters. This
    is an API change we didn't find any users and the API is
    declared unstable.
  + tests: Get properties all at once.
  + lfb-event: Add getter and setter for app-id. For applications
    this is figured out automatically but notification daemons and
    portal implementations might want to override it to have the
    per-app feedback profile picked up automatically.
- Drop 109.patch: Fixed upstream.
* Tue Mar 21 2023 Bjørn Lie <bjorn.lie@gmail.com>
- Add 109.patch: Fix build with glib-2.76.x
* Thu Mar 16 2023 Bjørn Lie <bjorn.lie@gmail.com>
- Update to version 0.1.0:
  * Make LED class more flexible. This will allow e.g. hw based
    implementations.
  * Trigger module load via udev. This avoids shipping another conf
    file.
- Changes from version 0.0.3:
  * spec: Separate system and application events
  * theme: add screen-shot event
  * Build and doc fixes
  * Update gmobile to 0.0.1
* Tue Jan 31 2023 Adrian Campos Garrido <adriancampos@teachelp.com>
- Update to version 0.0.2:
  * Support theme merging
  * Support user themes (`theme` gsetting)
  * Add theme validator
* Mon Dec  5 2022 Bjørn Lie <bjorn.lie@gmail.com>
- Update to version 0.0.1:
  * spec: Clarify feedback noisiness.
    The old wording could be interpreted as the feedback being
    discarded if it's noisier than the currently selected profile
    which is not the case. Also take the new per application
    settings into account.
  * lfb-event: Fix SPDX identifier. The library is LGPL-2.1+, not
    GPL-3.0+
  * test-lfb-integration: Test event state in async callback
  * lfb-event: Notify state change before completing async task
    lfb_event_trigger_feedback_async() internally uses a GTask to
    handle invocation of the user provided callback when
    g_task_return_*() is called.
    Currently GTask is completed first and only afterwards is the
    state of the event being updated which can lead to suprising
    behaviour.
  * feedback-manager: Use guint for event id.
    The ID is inserted into a hashtable below with GUINT_TO_POINTER
    and is set using self->next_id++ with next_id also being a
    guint, so change the type accordingly.
  * run: Make it easy to run under gdb and document usage.
    Having a fast and easy way to run the daemon under gdb can be
    beneficial for debugging. For that start the run script with
    FBD_GDB=1.
  * tests/integration: Indicate that callback quits main loop
    on_event_triggered() is renamed to on_event_triggered_quit()
    as we will shortly introduce a variant that does not quit.
  * tests/integration: Fix whitespace.
    Remove all tabs in favour of spaces and align function
    arguments.
  * tests/integration: Test that "feedback-ended" is propagated
    through LfbEvent. When no feedbacks are available (either
    because the requested event does not exist in the theme or
    because the system does not have required capabilities, e.g. no
    vibration motor or LED) feedbackd will emit a "feedback-ended"
    signal (with reason LFB_EVENT_STATE_REASON_NOT_FOUND) on DBus
    _before_ completing the event triggering DBus call. Since the
    DBus call was not completed yet, LfbEvent will not have learned
    it's ID yet and therefore not emit it's own "feedback-ended"
    GObject signal. In this case the test will timeout since the
    main loop is never quit.
  * feedback-manager: Complete DBus call before emitting
    "feedback-ended" See previous commit adding the test (that now
    passes) for the reasoning.
  * data: Drop outdated comment. We use the hints since some time,
    see lfb_event_set_feedback_profile().
  * lfb-event: Fix typo. It's LfbEvent
  * lfb-event: Use consistent prefix.
    Make async data use the same prefix.
  * Use g_error() when library is not initalized.
    This makes sure it still works when asserts are disabled.
  * lfb-event: Untabify
  * tests: Drop unused variable
  * tests/integration: Check the state of the ended event
  * manager: Use FEEDBACKD_UDEV_ATTR.
    It was introduced in
    a5b4f99 ("feedbackd: Support visual feedback via LEDs")
  * fbd: Fix indentation
* Mon Jan 31 2022 Bjørn Lie <bjorn.lie@gmail.com>
- Update to version 0.0.0+git20211018:
  * changelog: Bump changes line to a recent date too. It was still
    stuck at 2021-10-07. Fixup version too while at that.
- Changes from version 0.0.0+git20210426:
  * implement fbd_dev_sound_stop
    tracks the FdbFeedbackSounds in a GHashTable in the FdbDevSound
  * fbd-dev-sound: Don't leak GError
  * fbd-dev-sound: Don't warn when sound was cancelled. This is not
    an error per se.
  * lfb-event: Don't leak error.
    Don't leak the error values in the success case
  * tests: Run lfb_event_{get,set}_timeout
  * test-lfb-integration: Test invocation error
  * event: Make log-domain match source file name
  * tests: Run event tests too
  * event: Remove 'g' prefix from common C types.
    This matchs phosh's style.
  * fbd-event: Add sender property.
    This allows us to store the DBus sender
  * fbd-manager: Keep DBus sender around
  * feedback-manager: Track DBus clients and end feedbacks if they
    go away. When the client goes away we build up a list of events
    to end feedback for to make sure the hash table isn't modified
    in place. The alternative approach would be to disconnect the
    on_event_feedbacks_ended handler, duplicate the DBus signal
    emission and use g_hash_table_foreach_remove but that looks
    less robust than using the same code path.
  * Help emacs to indent properly
  * fbd-feedback-theme: Don't leak JSON node
  * fbd-dev-leds: Free actual FbdDevLed as well.
    So far we only unref'ed the device.
  * fbd-dev-leds: Don't leak enum_name
    g_enum_to_string() is transfer full.
  * fbd-dev-leds: Don't leak color
    g_ascii_strdown() creates a copy already.
  * fbd-dev-leds: Use automatic deep cleanup for device list.
    The current logic was fragile and failed e.g. when the
    FEEDBACKD_UDEV_VAL_LED would not match leaking the dev since
    the g_object_unref() at the end of the loop was never hit.
    Prevent that with automatic cleanup and explicitly ref'ing the
    devs we want to use.
  * fbd-dev-sound: Use correct cleanup for hash table.
    This went unnoticed since the manager forgot to cleanup
    this at all (see follow up commit).
  * fbd-feedback-manager: Make sure to dispose sound device as
    well. Fixes another leak.
  * fbd-feedback-manager: Don't leak config_path. It's assigned
    multiple times.
  * fbd-feedback-manager: Don't leak device list
  * fbd-ledctrl: fix matching of trigger in list of triggers.
    If the trigger is the last thing in
    sysfs_path/LED_TRIGGER_ATTR, then the new-line at the end of
    the output causes g_strv_contains to fail to find the trigger
    in the list. This strips the new-line before splitting the
    output into a list for searching.
  * README: Fix gdbus test command. It is TriggerFeedback and not
    Feedback (anymore). Also add a note how to end the feedback in
    order to prevent loss of hearing :-).
- Switch to buildtime tarball generation, use cpio format. Also
  switch to a explicit released tag, we like to know what we are
  getting is the same every time. At the same strip v prefix from
  version in upstream tag.
- Split out shared library and typelib in separate packages, and
  name them after their soname and typelibname, follow the SLPP!
- Drop unneeded systemd-devel BuildRequires and
  sound-theme-freedesktop Requires.
- Replace gcc with generic c_compiler BuildRequires.
* Tue Apr 20 2021 Adrian Campos Garrido <adriancampos@teachelp.com>
- Change feedbackd to video in udev rules
* Mon Apr 12 2021 Adrian Campos Garrido <adriancampos@teachelp.com>
- Add haptic support in feedback via udev rule
* Sun Apr 11 2021 Andrés B.S. <andresbs@opensuse.org>
- Update to tag 0.0.0+git20210125
* Mon Oct  5 2020 Adrian Campos Garrido <adriancampos@teachelp.com>
- Update feedbackd to add pinephone support
* Mon Sep 14 2020 Adrian Campos Garrido <adriancampos@teachelp.com>
- Fix problem with fbd-ledctrl file
* Fri Jul 24 2020 Adrian Campos Garrido <adriancampos@teachelp.com>
- Update feedbackd
* Fri Jun 19 2020 Adrian Campos Garrido <adriancampos@teachelp.com>
- Initial packaging
* Sun Jan 19 2020 Adrian Campos Garrido <adriancampos@teachelp.com>
- Update package
